(*
  PolytopeVolume.m: Volume Computation for Convex Polytopes
     
             Version 0.1 Beta
              July 28, 1993

           Copyright (c) 1993 by

  Komei Fukuda
  Graduate School of Systems Management
  University of Tsukuba, Tokyo
  3-29-1 Otsuka, Bunkyo-ku
  Tokyo, Japan 112
  +81-3-3942-6876
  fukuda@gssm.otsuka.tsukuba.ac.jp
*)

BeginPackage["PolytopeVolume`", "VertexEnum`"]


PolytopeVolume::usage = "PolytopeVolume[m,b] gives the volume of the convex polytope  P= {x : m x <= b and x>=0}, where m is an s*d matrix and b an s-vector."

Begin["`Private`"]    (* begin the private context *)

LastNonzero[v_List]:=Last[Select[v,Function[x,Rationalize[N[x],0.000001]!=0]]]

LexPositiveQ[m_?MatrixQ]:=
	Block[{j=1,dim=Dimensions[m],row,lpos=True},
		While[j<=dim[[1]] && lpos,
			row=m[[j]];
			lpos=(Rationalize[N[LastNonzero[row]],0.000001]>0);
			j=j+1
		];
		lpos
	]


(* 
Computation of the volume of the bounded cone
specified with a feasible nonbasis and a linear function
f(x)=cx. The volume will be evaluated iff the nonbasis is
lexico positive, ie, the basic solution is lexico feasible  
*)

ConeVolume[c_List,mat_List,b_List,nbindex_List]:=
	Block[{d=Dimensions[mat],mlarge,clarge,bindex,
		basis,binv,nbasis,cb,cn,bsol,rcosts,mtail},
		mlarge=
		Transpose[Join[Transpose[mat],IdentityMatrix[d[[1]]]]];
		clarge=Join[c,Table[0,{d[[1]]}]];
		bindex=Complement[Range[d[[1]]+d[[2]]],nbindex];
		basis=Transpose[Transpose[mlarge][[bindex]]];
		binv=Inverse[basis];
		nbasis=Transpose[Transpose[mlarge][[nbindex]]];
		cb=clarge[[bindex]];cn=clarge[[nbindex]];
		rcosts=cn-cb.binv.nbasis;
		bsol=binv.b;
		objval=cb.bsol;
		mtail=Transpose[Append[Transpose[binv],bsol]];
		If[LexPositiveQ[mtail],
			objval^d[[2]]/(Apply[Times,rcosts]*Abs[Det[basis]]*d[[2]]!),
			0
		]
	]

(* definition of the exported functions *)

PolytopeVolume[mat_?MatrixQ,vec_?VectorQ]:=
	Block[{c,d=Dimensions[mat],cvols,v,v2,vol},
		v=VertexEnumeration[mat,vec,SearchTree->True];
		Print["The set of vertices is ", Union[v[[1]]],"."];
		c=Table[Random[Integer,{1,1000}],{d[[2]]}];
		v2=Map[Take[#,d[[2]]]&,v[[2]]];
		cvols=Map[ConeVolume[c,mat,vec,#]&,v2];
		vol=Apply[Plus,cvols];
		Abs[vol]
	]


End[]         (* end the private context *)

EndPackage[]  (* end the package context *)